# encoding: utf-8
from django.conf.urls import include, url
from django.contrib import admin

from haystack.forms import FacetedSearchForm
from haystack.query import SearchQuerySet
from haystack.views import FacetedSearchView, SearchView, basic_search

admin.autodiscover()


urlpatterns = [
    url(r"^admin/", admin.site.urls),
    url(r"^$", SearchView(load_all=False), name="haystack_search"),
    url(
        r"^faceted/$",
        FacetedSearchView(
            searchqueryset=SearchQuerySet().facet("author"),
            form_class=FacetedSearchForm,
        ),
        name="haystack_faceted_search",
    ),
    url(r"^basic/$", basic_search, {"load_all": False}, name="haystack_basic_search"),
]

urlpatterns += [
    url(
        r"",
        include(("test_haystack.test_app_without_models.urls", "app-without-models")),
    )
]



# encoding: utf-8
import datetime
import os
from tempfile import mkdtemp
from unittest.mock import patch

import pysolr
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.management import call_command
from django.core.management.base import CommandError
from django.test import TestCase

from haystack import connections, constants, indexes
from haystack.utils.loading import UnifiedIndex

from ..core.models import MockModel, MockTag

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class SolrMockSearchIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(document=True, use_template=True)
    name = indexes.CharField(model_attr="author", faceted=True)
    pub_date = indexes.DateTimeField(model_attr="pub_date")

    def get_model(self):
        return MockModel

    def get_updated_field(self):
        return "pub_date"


class SolrMockTagSearchIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(document=True, model_attr="name")

    def get_model(self):
        return MockTag


class SolrMockSecretKeySearchIndex(indexes.SearchIndex, indexes.Indexable):
    Th3S3cr3tK3y = indexes.CharField(document=True, model_attr="author")

    def get_model(self):
        return MockModel


class ManagementCommandTestCase(TestCase):
    fixtures = ["base_data.json", "bulk_data.json"]

    def setUp(self):
        super(ManagementCommandTestCase, self).setUp()
        self.solr = pysolr.Solr(settings.HAYSTACK_CONNECTIONS["solr"]["URL"])

        # Stow.
        self.old_ui = connections["solr"].get_unified_index()
        self.ui = UnifiedIndex()
        self.smmi = SolrMockSearchIndex()
        self.ui.build(indexes=[self.smmi])
        connections["solr"]._index = self.ui

    def tearDown(self):
        connections["solr"]._index = self.old_ui
        super(ManagementCommandTestCase, self).tearDown()

    def verify_indexed_documents(self):
        """Confirm that the documents in the search index match the database"""

        res = self.solr.search("*:*", fl=["id"], rows=50)
        self.assertEqual(res.hits, 23)

        indexed_doc_ids = set(i["id"] for i in res.docs)
        expected_doc_ids = set(
            "core.mockmodel.%d" % i
            for i in MockModel.objects.values_list("pk", flat=True)
        )

        self.assertSetEqual(indexed_doc_ids, expected_doc_ids)

    def test_basic_commands(self):
        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", verbosity=0, commit=False)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", verbosity=0)
        self.verify_indexed_documents()

        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("rebuild_index", interactive=False, verbosity=0, commit=False)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("rebuild_index", interactive=False, verbosity=0, commit=True)
        self.verify_indexed_documents()

        call_command("clear_index", interactive=False, verbosity=0, commit=False)
        self.verify_indexed_documents()

    def test_remove(self):
        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", verbosity=0)
        self.verify_indexed_documents()

        # Remove several instances, two of which will fit in the same block:
        MockModel.objects.get(pk=1).delete()
        MockModel.objects.get(pk=2).delete()
        MockModel.objects.get(pk=8).delete()
        self.assertEqual(self.solr.search("*:*").hits, 23)

        # Plain ``update_index`` doesn't fix it.
        call_command("update_index", verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 23)

        # Remove without commit also doesn't affect queries:
        call_command(
            "update_index", remove=True, verbosity=0, batchsize=2, commit=False
        )
        self.assertEqual(self.solr.search("*:*").hits, 23)

        # … but remove with commit does:
        call_command("update_index", remove=True, verbosity=0, batchsize=2)
        self.assertEqual(self.solr.search("*:*").hits, 20)

    def test_age(self):
        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        start = datetime.datetime.now() - datetime.timedelta(hours=3)
        end = datetime.datetime.now()

        mock = MockModel.objects.get(pk=1)
        mock.pub_date = datetime.datetime.now() - datetime.timedelta(hours=2)
        mock.save()
        self.assertEqual(
            MockModel.objects.filter(pub_date__range=(start, end)).count(), 1
        )

        call_command("update_index", age=3, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 1)

    def test_age_with_time_zones(self):
        """Haystack should use django.utils.timezone.now"""
        from django.utils.timezone import now as django_now
        from haystack.management.commands.update_index import now as haystack_now

        self.assertIs(
            haystack_now,
            django_now,
            msg="update_index should use django.utils.timezone.now",
        )

        with patch("haystack.management.commands.update_index.now") as m:
            m.return_value = django_now()
            self.test_age()
            assert m.called

    def test_dates(self):
        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        start = datetime.datetime.now() - datetime.timedelta(hours=5, minutes=30)
        end = datetime.datetime.now() - datetime.timedelta(hours=2)

        mock_1 = MockModel.objects.get(pk=1)
        mock_1.pub_date = datetime.datetime.now() - datetime.timedelta(
            hours=5, minutes=1
        )
        mock_1.save()
        mock_2 = MockModel.objects.get(pk=2)
        mock_2.pub_date = datetime.datetime.now() - datetime.timedelta(hours=3)
        mock_2.save()
        mock_3 = MockModel.objects.get(pk=3)
        mock_3.pub_date = datetime.datetime.now() - datetime.timedelta(hours=1)
        mock_3.save()
        self.assertEqual(
            MockModel.objects.filter(pub_date__range=(start, end)).count(), 2
        )

        call_command(
            "update_index",
            start_date=start.isoformat(),
            end_date=end.isoformat(),
            verbosity=0,
        )
        self.assertEqual(self.solr.search("*:*").hits, 2)

    def test_multiprocessing(self):
        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", verbosity=2, workers=2, batchsize=5)
        self.verify_indexed_documents()

        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", verbosity=2, workers=2, batchsize=5, commit=False)
        self.assertEqual(self.solr.search("*:*").hits, 0)

    def test_build_schema_wrong_backend(self):

        settings.HAYSTACK_CONNECTIONS["whoosh"] = {
            "ENGINE": "haystack.backends.whoosh_backend.WhooshEngine",
            "PATH": mkdtemp(prefix="dummy-path-"),
        }

        connections["whoosh"]._index = self.ui
        self.assertRaises(
            ImproperlyConfigured, call_command, "build_solr_schema", using="whoosh"
        )

    def test_build_schema(self):

        # Stow.
        oldhdf = constants.DOCUMENT_FIELD
        oldui = connections["solr"].get_unified_index()
        oldurl = settings.HAYSTACK_CONNECTIONS["solr"]["URL"]

        try:
            needle = "Th3S3cr3tK3y"
            constants.DOCUMENT_FIELD = (
                needle
            )  # Force index to use new key for document_fields
            settings.HAYSTACK_CONNECTIONS["solr"]["URL"] = (
                settings.HAYSTACK_CONNECTIONS["solr"]["URL"].rsplit("/", 1)[0] + "/mgmnt"
            )

            ui = UnifiedIndex()
            ui.build(indexes=[SolrMockSecretKeySearchIndex()])
            connections["solr"]._index = ui

            rendered_file = StringIO()

            script_dir = os.path.realpath(os.path.dirname(__file__))
            conf_dir = os.path.join(
                script_dir, "server", "solr", "server", "solr", "mgmnt", "conf"
            )
            schema_file = os.path.join(conf_dir, "schema.xml")
            solrconfig_file = os.path.join(conf_dir, "solrconfig.xml")

            self.assertTrue(
                os.path.isdir(conf_dir), msg="Expected %s to be a directory" % conf_dir
            )

            call_command("build_solr_schema", using="solr", stdout=rendered_file)
            contents = rendered_file.getvalue()
            self.assertGreater(contents.find('name="%s' % needle), -1)

            call_command("build_solr_schema", using="solr", configure_directory=conf_dir)
            with open(schema_file) as s:
                self.assertGreater(s.read().find('name="%s' % needle), -1)
            with open(solrconfig_file) as s:
                self.assertGreater(s.read().find('name="df">%s' % needle), -1)

            self.assertTrue(os.path.isfile(os.path.join(conf_dir, "managed-schema.old")))

            call_command("build_solr_schema", using="solr", reload_core=True)

            os.rename(schema_file, "%s.bak" % schema_file)
            self.assertRaises(
                CommandError,
                call_command,
                "build_solr_schema",
                using="solr",
                reload_core=True,
            )

            call_command("build_solr_schema", using="solr", filename=schema_file)
            with open(schema_file) as s:
                self.assertGreater(s.read().find('name="%s' % needle), -1)
        finally:
            # reset
            constants.DOCUMENT_FIELD = oldhdf
            connections["solr"]._index = oldui
            settings.HAYSTACK_CONNECTIONS["solr"]["URL"] = oldurl


class AppModelManagementCommandTestCase(TestCase):
    fixtures = ["base_data", "bulk_data.json"]

    def setUp(self):
        super(AppModelManagementCommandTestCase, self).setUp()
        self.solr = pysolr.Solr(settings.HAYSTACK_CONNECTIONS["solr"]["URL"])

        # Stow.
        self.old_ui = connections["solr"].get_unified_index()
        self.ui = UnifiedIndex()
        self.smmi = SolrMockSearchIndex()
        self.smtmi = SolrMockTagSearchIndex()
        self.ui.build(indexes=[self.smmi, self.smtmi])
        connections["solr"]._index = self.ui

    def tearDown(self):
        connections["solr"]._index = self.old_ui
        super(AppModelManagementCommandTestCase, self).tearDown()

    def test_app_model_variations(self):
        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 25)

        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", "core", verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 25)

        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        with self.assertRaises(ImproperlyConfigured):
            call_command("update_index", "fake_app_thats_not_there")

        call_command("update_index", "core", "discovery", verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 25)

        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", "discovery", verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", "core.MockModel", verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 23)

        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", "core.MockTag", verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 2)

        call_command("clear_index", interactive=False, verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 0)

        call_command("update_index", "core.MockTag", "core.MockModel", verbosity=0)
        self.assertEqual(self.solr.search("*:*").hits, 25)



# encoding: utf-8
from django.contrib import admin

from haystack.admin import SearchModelAdmin

from .models import MockModel


class MockModelAdmin(SearchModelAdmin):
    haystack_connection = "solr"
    date_hierarchy = "pub_date"
    list_display = ("author", "pub_date")


admin.site.register(MockModel, MockModelAdmin)



# -*- coding: utf-8 -*-
import warnings

from django.conf import settings

import unittest
from haystack.utils import log as logging

warnings.simplefilter("ignore", Warning)


def setup():
    log = logging.getLogger("haystack")
    try:
        import elasticsearch

        if not ((2, 0, 0) <= elasticsearch.__version__ < (3, 0, 0)):
            raise ImportError
        from elasticsearch import Elasticsearch, exceptions
    except ImportError:
        log.error(
            "Skipping ElasticSearch 2 tests: 'elasticsearch>=2.0.0,<3.0.0' not installed."
        )
        raise unittest.SkipTest("'elasticsearch>=2.0.0,<3.0.0' not installed.")

    url = settings.HAYSTACK_CONNECTIONS["elasticsearch"]["URL"]
    es = Elasticsearch(url)
    try:
        es.info()
    except exceptions.ConnectionError as e:
        log.error("elasticsearch not running on %r" % url, exc_info=True)
        raise unittest.SkipTest("elasticsearch not running on %r" % url, e)



# encoding: utf-8
from django.conf.urls import url

from haystack.views import SearchView


class CustomPerPage(SearchView):
    results_per_page = 1


urlpatterns = [
    url(r"^search/$", CustomPerPage(load_all=False), name="haystack_search"),
    url(
        r"^search2/$",
        CustomPerPage(load_all=False, results_per_page=2),
        name="haystack_search",
    ),
]



#!/usr/bin/env python
# encoding: utf-8
import sys
from os.path import abspath, dirname

import nose


def run_all(argv=None):
    sys.exitfunc = lambda: sys.stderr.write("Shutting down....\n")

    # always insert coverage when running tests through setup.py
    if argv is None:
        argv = [
            "nosetests",
            "--with-coverage",
            "--cover-package=haystack",
            "--cover-erase",
            "--verbose",
        ]

    nose.run_exit(argv=argv, defaultTest=abspath(dirname(__file__)))


if __name__ == "__main__":
    run_all(sys.argv)



# encoding: utf-8
import haystack
from haystack.signals import RealtimeSignalProcessor

from django.apps import apps

from ..utils import check_solr

_old_sp = None


def setup():
    check_solr()
    global _old_sp
    config = apps.get_app_config("haystack")
    _old_sp = config.signal_processor
    config.signal_processor = RealtimeSignalProcessor(
        haystack.connections, haystack.connection_router
    )


def teardown():
    config = apps.get_app_config("haystack")
    config.signal_processor.teardown()
    config.signal_processor = _old_sp



#!/usr/bin/env python
# encoding: utf-8

from setuptools import setup

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools

    use_setuptools()
    from setuptools import setup

install_requires = ["Django>=2.2"]

tests_require = [
    "pysolr>=3.7.0",
    "whoosh>=2.5.4,<3.0",
    "python-dateutil",
    "geopy==0.95.1",
    "nose",
    "coverage",
    "requests",
]

setup(
    name="django-haystack",
    use_scm_version=True,
    description="Pluggable search for Django.",
    author="Daniel Lindsley",
    author_email="daniel@toastdriven.com",
    long_description=open("README.rst", "r").read(),
    url="http://haystacksearch.org/",
    packages=[
        "haystack",
        "haystack.backends",
        "haystack.management",
        "haystack.management.commands",
        "haystack.templatetags",
        "haystack.utils",
    ],
    package_data={
        "haystack": ["templates/panels/*", "templates/search_configuration/*"]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
    ],
    zip_safe=False,
    install_requires=install_requires,
    tests_require=tests_require,
    test_suite="test_haystack.run_tests.run_all",
)
